import arcpy
import math

sinuosity_expression = '''
import math
def getSinuosity(shape):
    length = shape.length
    d = math.sqrt((shape.firstPoint.X - shape.lastPoint.X) ** 2 +
                  (shape.firstPoint.Y - shape.lastPoint.Y) ** 2)
    return d / length
'''

class Toolbox:
    def __init__(self):
        self.alias  = "sample"
        self.label  = "Sinuosity Example"

        # List of tool classes associated with this toolbox
        self.tools = [CalculateSinuosity]

class CalculateSinuosity:
    def __init__(self):
        """Define the tool (tool name is the name of the class)"""
        self.label       = "Calculate Sinuosity"
        self.description = "Sinuosity measures the deviation of a " + \
                           "line from the shortest path, calculated " + \
                           "by dividing total length by shortest " + \
                           "possible path.  A straight feature will " + \
                           "have an sinuosity index of 1"

    def getParameterInfo(self):
        """Define parameter definitions"""

        # First parameter
        p0 = arcpy.Parameter()
        p0.displayName   = "Input Features"
        p0.name          = "in_features"
        p0.datatype      = "Feature Layer"
        p0.parameterType = "Required"
        p0.direction     = "Input"
        # Accept only polyline features
        p0.filter.list   = ["Polyline"]

        # Second parameter
        p1 = arcpy.Parameter()
        p1.displayName   = "Sinuosity Field"
        p1.name          = "sinuosity_field"
        p1.datatype      = "Field"
        p1.parameterType = "Optional"
        p1.direction     = "Input"
        #
        p1.value         = "sinuosity"

        # Third parameter
        p2 = arcpy.Parameter()
        p2.displayName   = "Output Features"
        p2.name          = "out_features"
        p2.datatype      = "Feature Layer"
        p2.parameterType = "Derived"
        p2.direction     = "Output"
        #
        p2.parameterDependencies = [p0.name]
        p2.schema.clone = True

        return [p0, p1, p2]


    def isLicensed(self):
        """Set whether tool is licensed to execute"""
        return True

    def updateParameters(self, parameters):
        """Modify the values and properties of parameters before internal
        validation is performed.  This method is called whenever a parameter
        has been changed."""

        fname = "sinuosity"
        if parameters[1].value:
            fname = parameters[1].value.value

        # Validate field name and update as necessary
        if parameters[0].altered and parameters[0].value:
            fname = arcpy.ValidateFieldName(fname, parameters[0].value)
            parameters[1].value = fname

        # Update the schema with the new field
        field = arcpy.Field()
        field.name = fname
        field.type = 'DOUBLE'
        parameters[2].schema.additionalFields = [field]

        return

    def updateMessages(self, parameters):
        """Modify the messages created by internal validation for each tool
        parameter.  This method is called after internal validation."""
        return

    def execute(self, parameters, messages):
        """The source code of the tool"""
        try:
            inFeatures  = parameters[0].valueAsText
            fieldName   = parameters[1].valueAsText

            if fieldName in ["#", "", None]:
                fieldName = "sinuosity"

            messages.AddMessage("Adding %s field" % fieldName)
            arcpy.AddField_management(inFeatures, fieldName, 'DOUBLE')

            messages.AddMessage("Calculating sinuosity")
            arcpy.CalculateField_management(inFeatures,
                                            fieldName,
                                            'getSinuosity(!shape!)',
                                            'PYTHON_9.3',
                                            sinuosity_expression)

        except Exception as e:
            messages.AddErrorMessage(e[0])



